@@ -6,36 +6,48 @@ import unittest
 import sys
 sys.path.insert(0, "../")
 
-from gi.repository import Gdk
-from gi.repository import Gtk
+from gi.repository import GLib
 
-import gi.pygtkcompat
+try:
+    from gi.repository import Pango
+    from gi.repository import Atk
+    from gi.repository import Gdk
+    from gi.repository import Gtk
+    (Atk, Gtk, Pango)  # pyflakes
 
-gi.pygtkcompat.enable()
-gi.pygtkcompat.enable_gtk(version='3.0')
+    import gi.pygtkcompat
 
-import atk
-import pango
-import pangocairo
-import gtk
-import gtk.gdk
+    gi.pygtkcompat.enable()
+    gi.pygtkcompat.enable_gtk(version='3.0')
+
+    import atk
+    import pango
+    import pangocairo
+    import gtk
+    import gtk.gdk
+except ImportError:
+    Gtk = None
 
 
+@unittest.skipUnless(Gtk, 'Gtk not available')
 class TestATKCompat(unittest.TestCase):
     def testObject(self):
         self.assertTrue(hasattr(atk, 'Object'))
 
 
+@unittest.skipUnless(Gtk, 'Gtk not available')
 class TestPangoCompat(unittest.TestCase):
     def testLayout(self):
         self.assertTrue(hasattr(pango, 'Layout'))
 
 
+@unittest.skipUnless(Gtk, 'Gtk not available')
 class TestPangoCairoCompat(unittest.TestCase):
     def testErrorUnderlinePath(self):
         self.assertTrue(hasattr(pangocairo, 'error_underline_path'))
 
 
+@unittest.skipUnless(Gtk, 'Gtk not available')
 class TestGTKCompat(unittest.TestCase):
     def testButtons(self):
         self.assertEqual(Gdk._2BUTTON_PRESS, 5)
@@ -90,7 +102,11 @@ class TestGTKCompat(unittest.TestCase):
         liststore.append((1, 'One'))
         liststore.append((2, 'Two'))
         liststore.append((3, 'Three'))
+        # might cause a Pango warning, do not break on this
+        old_mask = GLib.log_set_always_fatal(
+            GLib.LogLevelFlags.LEVEL_CRITICAL | GLib.LogLevelFlags.LEVEL_ERROR)
         combo = gtk.ComboBoxEntry(model=liststore)
+        GLib.log_set_always_fatal(old_mask)
         combo.set_text_column(1)
         combo.set_active(0)
         self.assertEqual(combo.get_text_column(), 1)
